@@ -2,15 +2,15 @@
 ## (C)2006 Simon Urbanek <simon.urbanek@r-project.org>
 ## For license terms see DESCRIPTION and/or LICENSE
 ##
-## $Id: call.R 570 2011-07-11 15:28:40Z urbanek $
+## $Id$
 
 # create a new object
-.jnew <- function(class, ..., check=TRUE, silent=!check) {
+.jnew <- function(class, ..., check=TRUE, silent=!check, class.loader=NULL) {
   class <- gsub("\\.", "/", as.character(class)) # allow non-JNI specifiation
   # TODO: should this do "S" > "java/lang/String", ... like .jcall
   
   if (check) .jcheck(silent=TRUE)
-  o<-.External(RcreateObject, class, ..., silent=silent)
+  o<-.External(RcreateObject, class, ..., silent=silent, class.loader=class.loader)
   if (check) .jcheck(silent=silent)
   if (is.null(o)) {
   	  if (!silent) {
@@ -64,6 +64,12 @@
     return(.Call(RgetFloatArrayCont, jobj))
   else if (sig=="[Ljava/lang/String;")
     return(.Call(RgetStringArrayCont, jobj))
+  else if (sig=="[Ljava/lang/Double;" && simplify) {
+    obj@jclass <- sig; return(.jcall("RJavaArrayTools", "[D", "unboxDoubles", obj)) }
+  else if (sig=="[Ljava/lang/Integer;" && simplify) {
+    obj@jclass <- sig; return(.jcall("RJavaArrayTools", "[I", "unboxIntegers", obj)) }
+  else if (sig=="[Ljava/lang/Boolean;" && simplify) {
+    obj@jclass <- sig; return(as.logical(.jcall("RJavaArrayTools", "[I", "unboxBooleans", obj))) }
   else if (substr(sig,1,2)=="[L")
     return(lapply(.Call(RgetObjectArrayCont, jobj),
                   function(x) new("jobjRef", jobj=x, jclass=substr(sig, 3, nchar(sig)-1)) ))
@@ -317,7 +323,7 @@ is.jnull <- function(x) {
   else
     try(a <- .jcall("java/lang/Class","Ljava/lang/Class;","forName",cl,check=FALSE))
   # this is really .jcheck but we don't want it to appear on the call stack
-  .C(RJavaCheckExceptions, silent, FALSE, PACKAGE = "rJava")
+  .Call(RJavaCheckExceptions, silent)
   if (!silent && is.jnull(a)) stop("class not found")
   a
 }
