@@ -2,7 +2,7 @@
 ## (C)2006 Simon Urbanek <simon.urbanek@r-project.org>
 ## For license terms see DESCRIPTION and/or LICENSE
 ##
-## $Id: jinit.R 570 2011-07-11 15:28:40Z urbanek $
+## $Id$
 
 .check.JVM <- function() 
     .Call(RJava_checkJVM)
@@ -22,8 +22,7 @@
       return(0)
     }
   }
-  
-  
+
   ## determine path separator
   path.sep <- .Platform$path.sep
 
@@ -49,10 +48,21 @@
   if (is.character(.Platform$r_arch) && nzchar(.Platform$r_arch) && length(grep("-Dr.arch", parameters, fixed=TRUE)) == 0L)
     parameters <- c(paste("-Dr.arch=/", .Platform$r_arch, sep=''), as.character(parameters))
 
+  ## unfortunately Sys/setlocale()/Sys.getlocale() have incompatible interfaces so there
+  ## is no good way to get/set locales -- so we have to hack around it ...
+  locale.list <- c("LC_COLLATE", "LC_CTYPE", "LC_MONETARY", "LC_NUMERIC", "LC_TIME", "LC_MESSAGES", "LC_PAPER", "LC_MEASUREMENT")
+  locales <- sapply(locale.list, Sys.getlocale)
+  loc.sig <- Sys.getlocale()
+
   #cat(">> init CLASSPATH =",classpath,"\n")
   #cat(">> boot class path: ", boot.classpath,"\n")
   # call the corresponding C routine to initialize JVM
-  xr<-.External(RinitJVM, boot.classpath, parameters)
+  xr <- .External(RinitJVM, boot.classpath, parameters)
+
+  ## we have to re-set the locales right away
+  suppressWarnings(try(if (!identical(Sys.getlocale(), loc.sig)) for (i in names(locales)) try(Sys.setlocale(i, locales[i]), silent=TRUE),
+      silent=TRUE))
+
   if (xr==-1) stop("Unable to initialize JVM.")
   if (xr==-2) stop("Another VM is already running and rJava was unable to attach to that VM.")
   # we'll handle xr==1 later because we need fully initialized rJava for that
@@ -75,21 +85,11 @@
   assign(".jclassClass", .jcall("java/lang/Class","Ljava/lang/Class;","forName","java.lang.Class"), .env)
   assign(".jclassString", .jcall("java/lang/Class","Ljava/lang/Class;","forName","java.lang.String"), .env)
 
-  ic <- .jcall("java/lang/Class","Ljava/lang/Class;","forName","java.lang.Integer")
-  f<-.jcall(ic,"Ljava/lang/reflect/Field;","getField", "TYPE")
-  assign(".jclass.int", .jcast(.jcall(f,"Ljava/lang/Object;","get",.jcast(ic,"java/lang/Object")),"java/lang/Class"), .env)
-  ic <- .jcall("java/lang/Class","Ljava/lang/Class;","forName","java.lang.Double")
-  f<-.jcall(ic,"Ljava/lang/reflect/Field;","getField", "TYPE")
-  assign(".jclass.double", .jcast(.jcall(f,"Ljava/lang/Object;","get",.jcast(ic,"java/lang/Object")),"java/lang/Class"), .env)
-  ic <- .jcall("java/lang/Class","Ljava/lang/Class;","forName","java.lang.Float")
-  f<-.jcall(ic,"Ljava/lang/reflect/Field;","getField", "TYPE")
-  assign(".jclass.float", .jcast(.jcall(f,"Ljava/lang/Object;","get",.jcast(ic,"java/lang/Object")),"java/lang/Class"), .env)
-  ic <- .jcall("java/lang/Class","Ljava/lang/Class;","forName","java.lang.Boolean")
-  f<-.jcall(ic,"Ljava/lang/reflect/Field;","getField", "TYPE")
-  assign(".jclass.boolean", .jcast(.jcall(f,"Ljava/lang/Object;","get",.jcast(ic,"java/lang/Object")),"java/lang/Class"), .env)
-  ic <- .jcall("java/lang/Class","Ljava/lang/Class;","forName","java.lang.Void")
-  f<-.jcall(ic,"Ljava/lang/reflect/Field;","getField", "TYPE")
-  assign(".jclass.void", .jcast(.jcall(f,"Ljava/lang/Object;","get",.jcast(ic,"java/lang/Object")),"java/lang/Class"), .env)
+  assign(".jclass.int", .jfield("java/lang/Integer", "Ljava/lang/Class;", "TYPE"), .env)
+  assign(".jclass.double", .jfield("java/lang/Double", "Ljava/lang/Class;", "TYPE"), .env)
+  assign(".jclass.float", .jfield("java/lang/Float", "Ljava/lang/Class;", "TYPE"), .env)
+  assign(".jclass.boolean", .jfield("java/lang/Boolean", "Ljava/lang/Class;", "TYPE"), .env)
+  assign(".jclass.void", .jfield("java/lang/Void", "Ljava/lang/Class;", "TYPE"), .env)
 
   ## if NOAWT is set, set AWT to headless
   if (nzchar(Sys.getenv("NOAWT"))) .jcall("java/lang/System","S","setProperty","java.awt.headless","true")
