@@ -75,6 +75,8 @@ HIDE void profReport(char *fmt, ...) {
 #endif
 
 static void JRefObjectFinalizer(SEXP ref) {
+    if (java_is_dead) return;
+
   if (TYPEOF(ref)==EXTPTRSXP) {
     JNIEnv *env=getJNIEnv();
     jobject o = R_ExternalPtrAddr(ref);
@@ -196,6 +198,18 @@ static void strcats(sig_buffer_t *sig, const char *add) {
   sig->len += l;
 }
 
+/* call strcats() but also convert class names to JNI notation */
+static void strcats_conv(sig_buffer_t *sig, const char *add) {
+  int ol = sig->len, nl;
+  strcats(sig, add);
+  nl = sig->len;
+  while (ol < nl) {
+    if (sig->sig[ol] == '.')
+      sig->sig[ol] = '/';
+    ol++;
+  }
+}
+
 /* initialize a signature buffer */
 HIDE void init_sigbuf(sig_buffer_t *sb) {
   sb->len = 0;
@@ -231,27 +245,36 @@ static int Rpar2jvalue(JNIEnv *env, SEXP par, jvalue *jpar, sig_buffer_t *sig, i
     if (TYPEOF(e)==STRSXP) {
       _dbg(rjprintf(" string vector of length %d\n",LENGTH(e)));
       if (LENGTH(e)==1) {
-	strcats(sig,"Ljava/lang/String;");
-	addtmpo(tmpo, jpar[jvpos++].l=newString(env, CHAR_UTF8(STRING_ELT(e,0))));
+	  SEXP sv = STRING_ELT(e, 0);
+	  strcats(sig,"Ljava/lang/String;");
+	  if (sv == R_NaString) {
+	      addtmpo(tmpo, jpar[jvpos++].l = 0);
+	  } else {
+	      addtmpo(tmpo, jpar[jvpos++].l = newString(env, CHAR_UTF8(sv)));
+	  }
       } else {
-	int j=0;
-	jobjectArray sa=(*env)->NewObjectArray(env, LENGTH(e), javaStringClass, 0);
-	_mp(MEM_PROF_OUT("  %08x LNEW string[%d]\n", (int) sa, LENGTH(e)))
-	if (!sa) {
-	  fintmpo(tmpo);
-	  error("unable to create string array.");
-	  return -1;
-	}
-	addtmpo(tmpo, sa);
-	while (j<LENGTH(e)) {
-	  jobject s=newString(env, CHAR_UTF8(STRING_ELT(e,j)));
-	  _dbg(rjprintf (" [%d] \"%s\"\n",j,CHAR_UTF8(STRING_ELT(e,j))));
-	  (*env)->SetObjectArrayElement(env, sa, j, s);
-	  if (s) releaseObject(env, s);
-	  j++;
-	}
-	jpar[jvpos++].l=sa;
-	strcats(sig,"[Ljava/lang/String;");
+	  int j = 0;
+	  jobjectArray sa = (*env)->NewObjectArray(env, LENGTH(e), javaStringClass, 0);
+	  _mp(MEM_PROF_OUT("  %08x LNEW string[%d]\n", (int) sa, LENGTH(e)))
+	  if (!sa) {
+	      fintmpo(tmpo);
+	      error("unable to create string array.");
+	      return -1;
+	  }
+	  addtmpo(tmpo, sa);
+	  while (j < LENGTH(e)) {
+	      SEXP sv = STRING_ELT(e,j);
+	      if (sv == R_NaString) {
+	      } else {
+		  jobject s = newString(env, CHAR_UTF8(sv));
+		  _dbg(rjprintf (" [%d] \"%s\"\n",j,CHAR_UTF8(sv)));
+		  (*env)->SetObjectArrayElement(env, sa, j, s);
+		  if (s) releaseObject(env, s);
+	      }
+	      j++;
+	  }
+	  jpar[jvpos++].l = sa;
+	  strcats(sig,"[Ljava/lang/String;");
       }
     } else if (TYPEOF(e)==RAWSXP) {
       _dbg(rjprintf(" raw vector of length %d\n", LENGTH(e)));
@@ -364,9 +387,9 @@ static int Rpar2jvalue(JNIEnv *env, SEXP par, jvalue *jpar, sig_buffer_t *sig, i
 	}
 	if (jc) {
 	  if (*jc!='[') { /* not an array, we assume it's an object of that class */
-	    strcats(sig,"L"); strcats(sig,jc); strcats(sig,";");
+	    strcats(sig,"L"); strcats_conv(sig,jc); strcats(sig,";");
 	  } else /* array signature is passed as-is */
-	    strcats(sig,jc);
+	    strcats_conv(sig,jc);
 	} else
 	  strcats(sig,"Ljava/lang/Object;");
 	jpar[jvpos++].l=o;
@@ -442,7 +465,7 @@ REPE SEXP RcallMethod(SEXP par) {
   }
 #endif
   if (clnam)
-    cls = findClass(env, clnam);
+    cls = findClass(env, clnam, oClassLoader);
   else
     cls = objectClass(env,o);
   if (!cls)
@@ -675,7 +698,7 @@ REPE SEXP RcreateObject(SEXP par) {
   sig_buffer_t sig;
   jvalue jpar[maxJavaPars];
   jobject tmpo[maxJavaPars+1];
-  jobject o;
+  jobject o, loader = 0;
   JNIEnv *env=getJNIEnv();
 
   if (TYPEOF(p)!=LISTSXP) {
@@ -701,12 +724,25 @@ REPE SEXP RcreateObject(SEXP par) {
     if (TAG(p) && isSymbol(TAG(p))) {
       if (TAG(p)==install("silent") && isLogical(CAR(p)) && LENGTH(CAR(p))==1)
 	silent=LOGICAL(CAR(p))[0];
-    }
-    p=CDR(p);
-  }
 
+      /* class.loader */
+      if (TAG(p)==install("class.loader")) {
+	SEXP e = CAR(p);
+	if (TYPEOF(e) == S4SXP && IS_JOBJREF(e)) {
+	  SEXP sref = GET_SLOT(e, install("jobj"));
+	  if (sref && TYPEOF(sref) == EXTPTRSXP) {
+	    jverify(sref);
+	    loader = (jobject)EXTPTR_PTR(sref);
+	  }
+	} else if (e != R_NilValue)
+	  Rf_error("invalid class.loader");
+      }
+    }
+    p = CDR(p);
+  }
+  if (!loader) loader = oClassLoader;
 BEGIN_RJAVA_CALL
-  o = createObject(env, class, sig.sig, jpar, silent);
+  o = createObject(env, class, sig.sig, jpar, silent, loader);
 END_RJAVA_CALL
   done_sigbuf(&sig);
   Rfreejpars(env, tmpo);
@@ -873,7 +909,11 @@ REPC SEXP RcreateArray(SEXP ar, SEXP cl) {
 	jcharArray a = newCharArrayI(env, INTEGER(ar), LENGTH(ar));
 	if (!a) error("unable to create a char array");
 	return new_jarrayRef(env, a, "[C" ); 
-	  } else {
+      } else if (inherits(ar, "jshort")) {
+	jshortArray a = newShortArrayI(env, INTEGER(ar), LENGTH(ar));
+	if (!a) error("unable to create a short integer array");
+	return new_jarrayRef(env, a, "[S");
+      } else {
 	jintArray a = newIntArray(env, INTEGER(ar), LENGTH(ar));
 	if (!a) error("unable to create an integer array");
 	return new_jarrayRef(env, a, "[I");
@@ -900,11 +940,14 @@ REPC SEXP RcreateArray(SEXP ar, SEXP cl) {
       jobjectArray a = (*env)->NewObjectArray(env, LENGTH(ar), javaStringClass, 0);
       int i = 0;
       if (!a) error("unable to create a string array");
-      while (i<LENGTH(ar)) {
-	jobject so = newString(env, CHAR_UTF8(STRING_ELT(ar, i)));
-	(*env)->SetObjectArrayElement(env, a, i, so);
-	releaseObject(env, so);
-	i++;
+      while (i < LENGTH(ar)) {
+	  SEXP sa = STRING_ELT(ar, i);
+	  if (sa != R_NaString) {
+	      jobject so = newString(env, CHAR_UTF8(sa));
+	      (*env)->SetObjectArrayElement(env, a, i, so);
+	      releaseObject(env, so);
+	  }
+	  i++;
       }
       return new_jarrayRef(env, a, "[Ljava/lang/String;");
     }
@@ -935,7 +978,7 @@ REPC SEXP RcreateArray(SEXP ar, SEXP cl) {
       if (TYPEOF(cl)==STRSXP && LENGTH(cl)>0) {
 	const char *cname = CHAR_UTF8(STRING_ELT(cl, 0));
 	if (cname) {
-	  ac = findClass(env, cname);
+	  ac = findClass(env, cname, oClassLoader);
 	  if (!ac)
 	    error("Cannot find class %s.", cname);
 	  if (strlen(cname)<253) {
